# Targets:
# "godi_build": 
#   - recompiles all explicitly mentioned packages unconditionally
#   - recompiles all dependent packages if needed
# "godi_update"
#   - updates the mentioned packages, or all packages if LIST=""
#
# LIST: This variable is passed by the shell script and contains the
# package arguments
#
# FORCE=1
#
# cwd is pkgsrcdir
#
# ----------------------------------------------------------------------
# Written by Gerd Stolpmann

.include "${.PARSEDIR}/bsd.pkg.mk"

.SILENT:

SELF:=${.PARSEDIR}/godi_build.mk
AVAIL_DIR=${LOCALBASE}/build

.PHONY: godi_update
godi_update: godi_download_available
.	for pkg in ${LIST}
	  : ; \
          { \
	    pbase="${pkg:C/-[0-9][^-]*$//1}"; \
	    pdir=""; \
	    pversion=""; \
	    while read name dir version; do \
		if [ "X$$name" = "X$$pbase" ]; then \
		    pversion="$$version"; \
		    pdir="$$dir"; \
		fi; \
	    done; \
	    if [ -z "$$pdir" ]; then \
		echo "No such package: ${pkg}" >&2; \
		exit 1; \
	    fi; \
	    if [ "X$$pbase" = "X${pkg}" ]; then \
	        ${MAKE} -f ${SELF} godi_update_this \
		    "pkg=$$pbase" "dir=$$pdir" "version=$$pversion" </dev/null; \
	    else \
		version="${pkg:C/^.*-([0-9][^-]*)$/\1/1}"; \
	    	${MAKE} -f ${SELF} godi_install_this \
		        "pkg=$$pbase" "dir=$$pdir" "version=$$version" </dev/null; \
	    fi; \
	  } <${AVAIL_DIR}/available.current
.	endfor
.	if empty(LIST)
	    : ; \
	    { while read name dir version; do \
	        ${MAKE} -f ${SELF} godi_update_this \
		    "pkg=$$name" "dir=$$dir" "version=$$version" </dev/null; \
	      done; \
            } <${AVAIL_DIR}/available.current
.	endif

.PHONY: godi_update_this
godi_update_this:
	echo "===> Checking ${pkg}..."
	if [ -d "${LOCALBASE}/build/${dir}/${pkg}" ]; then \
	    inst_version=`cd ${LOCALBASE}/build/${dir}/${pkg} && ${MAKE} show-var VARNAME=PKGNAME | sed -e 's/^.*-\([0-9][^-]*\)$$/\1/'`; \
	    newer=0; \
	    if godi_admin pmatch 'x<${version}' "x-$${inst_version}"; then \
		newer=1; \
	    elif godi_admin pmatch 'x==${version}' "x-$${inst_version}"; then \
	        version_rev=`echo '${version}' | grep godi | sed -e 's/^.*godi//'`; \
	        inst_version_rev=`echo "$${inst_version}" | grep godi | sed -e 's/^.*godi//'`; \
	        if [ "0$$version_rev" -gt "0$$inst_version_rev" ]; then \
		    newer=1; \
		fi; \
	    fi; \
	    if [ "$$newer" -gt 0 ]; then \
		${MAKE} -f ${SELF} godi_install_this \
		    "pkg=${pkg}" "dir=${dir}" "version=${version}"; \
	    fi; \
	else \
		${MAKE} -f ${SELF} godi_install_this \
		    "pkg=${pkg}" "dir=${dir}" "version=${version}"; \
	fi

.PHONY: godi_install_this
godi_install_this:
	echo "===> Updating build instructions of ${pkg} for version ${version}"
	${MKDIR} "${LOCALBASE}/build/${dir}"
	${MKDIR} "${LOCALBASE}/build/buildfiles"
	cd "${LOCALBASE}/build/buildfiles" && \
	${FETCH_CMD} ${FETCH_BEFORE_ARGS} ${GODI_BUILD_SITE}/${pkg}-${version}.build.tgz ${FETCH_AFTER_ARGS} && \
	cd "${LOCALBASE}/build/${dir}" && \
	${RM} -rf "${pkg}" && \
	${GTAR} xzf "${LOCALBASE}/build/buildfiles/${pkg}-${version}.build.tgz" "${pkg}"

.PHONY: godi_download_available
godi_download_available:
	echo "===> Retrieving new package list..."
	cd ${AVAIL_DIR}; \
	${FETCH_CMD} ${FETCH_BEFORE_ARGS} ${GODI_BUILD_SITE}/available.new ${FETCH_AFTER_ARGS}
	cd ${AVAIL_DIR}; mv available.new available.current

## FIXME: godi_build does not cope with package revisions correctly!

.PHONY: godi_build
godi_build:
	echo "===> Analyzing dependencies and planning the build process..."
	tty=`tty -s && tty; true`; \
	${AWK} -v "localbase=${LOCALBASE}" \
	       -v "pkglist_to_update=${LIST}" \
	       -v "force=${FORCE}" \
	       -f ${LOCALBASE}/build/mk/scripts/godi_buildplan.awk | \
	{ read pkglist_to_delete; \
	  read pkglist_to_update; \
	  if [ -n "$$pkglist_to_delete" ]; then \
	      echo "     First these packages will be deleted:"; \
	      for p in $$pkglist_to_delete; do \
		  echo "     - " $$p; \
	      done; \
	  fi; \
	  if [ -n "$$pkglist_to_update" ]; then \
	      echo "     These packages will be built (or rebuilt):"; \
	      for p in $$pkglist_to_update; do \
		  echo "     - " $$p; \
		  pdir=`echo $$p | sed -e 's/-.*$$//'`; \
		  if [ ! -d "${LOCALBASE}/build/$$pdir/$$p" ]; then \
		      echo "          *** Package does not exist ***"; \
		  fi; \
		  if [ -f "${LOCALBASE}/build/$$pdir/$$p/BUILDMSG" ]; then \
		      echo "          *** Build note: ***"; \
		      sed -e 's/^/          : /' \
			  "${LOCALBASE}/build/$$pdir/$$p/BUILDMSG"; \
		  fi; \
	      done; \
	  fi; \
	  if [ -z "$$pkglist_to_delete" -a -z "$$pkglist_to_update" ]; then \
	      echo "     Nothing to do!"; \
	      exit 0; \
	  fi; \
	  if [ -n "$$tty" ]; then \
	      echo "===> Press [Return] to start, [Ctrl-C] to abort: "; \
	      printf ">>>> "; \
	      read dummy < $$tty; \
          fi; \
	  for p in $$pkglist_to_delete; do \
	      echo "=> Deleting $$p"; \
	      ${LOCALBASE}/sbin/godi_delete $$p; \
	  done; \
	  for p in $$pkglist_to_update; do \
	      echo "===> Building $$p"; \
	      pdir=`echo $$p | sed -e 's/-.*$$//'`; \
	      ( cd "${LOCALBASE}/build/$$pdir/$$p" && \
	          ${MAKE} clean && \
		  ${MAKE} package && \
		  ${MAKE} clean ) || exit; \
	  done; \
	  echo "!!!! godi_build done"; \
        }


## FIXME: godi_upgrade does not cope with package revisions correctly!

.PHONY: godi_upgrade
godi_upgrade:
	list=` \
          { \
	    while read name dir version; do \
	        echo "===> Checking package $$name" >&2; \
	        avail_version=\`cd ${LOCALBASE}/build/$$dir/$$name && ${MAKE} show-var VARNAME=PKGNAME | sed -e 's/^.*-\\([0-9][^-]*\\)$$/\\1/'\`; \
	        if godi_info -q -e "$$name" && \
                   godi_info -q -e "$$name<$$avail_version"; then \
	            echo "$$name"; \
	        fi; \
	    done; \
          } <${AVAIL_DIR}/available.current`; \
	if test -n "$$list"; then \
	    ${MAKE} -f ${SELF} godi_build "LIST=$$list"; \
	else \
	    echo "---- Packages are up-to-date"; \
	fi
